// Main.cpp
//Main flow to get symmetry planes
//from a graph-like representation of a polyhedrical shape

//Based on:
//    Plumed R., Company P. and Varley P.A.C.
//    "Detecting mirror symmetry in single-view wireframe sketches of polyhedral shapes"
//    Computers & Graphics, under review
//

#include "stdafx.h"
#include <vector>

#include "Tools_Vector.h"
#include "Tools_Geometry.h"
#include "CueSymmetry.h"


int _tmain(int argc, _TCHAR* argv[])
{
	//Assign vectors
	//std::vector <std::vector <long>> FinalFaceAxesU;
	//std::vector <std::vector <long>> FinalFaceAxesV;

	std::vector<std::vector<std::vector<long>>> FinalSymPlanes;
	std::vector<std::vector<double>> FinalSymMerits;

	//Find symmetry
	//----------
	CCueSymmetry Sym;
	Sym.GetBilateralSymmetryPlanes(
								   //FinalFaceAxesU,
								   //FinalFaceAxesV,
								   FinalSymPlanes,
								   FinalSymMerits);

	//Print output
	//------------
	long SubgraphCount= FinalSymPlanes.size();
	printf("Number of subgraphs= %3d \n", SubgraphCount);

	for(long NumGraph= 0; NumGraph<SubgraphCount; NumGraph++){
		printf("Subgraph number %3d \n", NumGraph);

		int SymCount= FinalSymPlanes[NumGraph].size();
		printf("Number of symmetry planes= %3d \n", SymCount);
		for(long i= 0; i<SymCount; i++){
			printf("Symmetry plane number %3d \n", i);
			long size= FinalSymPlanes[NumGraph][i].size();
			for(long j= 0; j<(long)FinalSymPlanes[NumGraph][i].size(); j++){
				long side= FinalSymPlanes[NumGraph][i][j];
				if(side >= 0)
					printf("Vertex %d\n", side);
				else
					printf("Edge %d\n", -side-1);
			}
			printf("Merit %f\n", FinalSymMerits[NumGraph][i]);
		}
	}

	printf("\n");


	system("PAUSE");
	return 0;
}
